function varargout = wiz(varargin)
% Display locations of the Bernstein Paper Atlas
%
% Vlad Atanasiu * 09/10/2007
% Last revision > 21/06/2008
%
% atanasiu@mit.edu
% http://mywebpage.netscape.com/atanasiuvlad/

% ----------------------------------
% INITIALISE GUI
% ----------------------------------

% WIZ M-file_menu for wiz.fig
%      WIZ, by itself, creates a new WIZ or raises the existing
%      singleton*.
%
%      H = WIZ returns the handle to a new WIZ or the handle to
%      the existing singleton*.
%
%      WIZ('CALLBACK',hObject,eventData,handles,...) calls the local
%      function named CALLBACK in WIZ.M with the given input arguments.
%
%      WIZ('Property','Value',...) creates a new WIZ or raises the
%      existing singleton*.  Starting from the left, property value pairs are
%      applied to the GUI before wiz_OpeningFunction gets called.  An
%      unrecognized property name or invalid value makes property application
%      stop.  All inputs are passed to wiz_OpeningFcn via varargin.
%
%      *See GUI Options on GUIDE's Tools menu.  Choose "GUI allows only one
%      instance to run (singleton)".
%
% See also: GUIDE, GUIDATA, GUIHANDLES

% Edit the above text to modify the response to help wiz

% Last Modified by GUIDE v2.5 17-Jun-2008 15:27:02
% global handles

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @wiz_OpeningFcn, ...
                   'gui_OutputFcn',  @wiz_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

global TogOnCol TogOnBkCol TogOffCol TogOffBkCol
TogOnCol = [1 1 1];
TogOnBkCol = [1 0 0];
TogOffCol = [1 1 1];
TogOffBkCol = [0.8 0.75 0.67];


% ----------------------------------
% LOAD GEODATA
% ----------------------------------
% --- Executes on button press in Map_pushbutton.
function LoadData_pushbutton_Callback(hObject, eventdata, handles)
% hObject    handle to Map_pushbutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% read data
global geo index ...
    TogOnCol TogOnBkCol TogOffCol TogOffBkCol

set(handles.LoadData_pushbutton,'String','Loading...',...
    'ForegroundColor',TogOnCol,'BackgroundColor',TogOnBkCol);
drawnow

% get complete name of georeferences file
t = dir;
for k = 1:size(t,1)
    if isempty(strfind(t(k).name,'dalek')) == 0
        t = t(k).name;
        break
    end
end
[num, txt, table] = xlsread(t);

k = size(txt,1);
geo = struct(...
    'locations_names_source',txt(2:k,3),...
    'locations_names',txt(2:k,4),...
	'locations_latitude',table(2:k,5),...
	'locations_longitude',table(2:k,6),...
    'countries_names',txt(2:k,8),...
    'regions1_names',txt(2:k,9),...
    'regions2_names',txt(2:k,10),...
    'regions3_names',txt(2:k,11),...
    'use_briquet',txt(2:k,12),...
    'repositories_briquet',txt(2:k,13),...
    'use_gw',txt(2:k,14),...
    'repositories_gw',txt(2:k,15),...
    'context_ibph',txt(2:k,16),...
    'use_istc',txt(2:k,17),...
    'use_likhachev',txt(2:k,18),...
    'repositories_likhachev',txt(2:k,19),...
    'use_pf',txt(2:k,20),...
    'use_po',txt(2:k,21),...
    'repositories_piccard',txt(2:k,22),...
    'use_wilc',txt(2:k,23),...
    'repositories_wilc',txt(2:k,24),...
    'use_wzma',txt(2:k,25),...
    'repositories_wzma',txt(2:k,26),...
    'context_bir',txt(2:k,27)...
    );

clear num txt table

% de-sparsify data: remove empty cells:
% find row indices of rows w/o place coordinates:
% remove header, change format, mark empty cells,...
% get indices, cut rows
geo(find(isnan([geo.locations_latitude]'))) = [];

% % aggregate data: remove duplicates
% [b, m, n] = unique(strcat({geo.source}',{geo.infotype}',...
%     [num2str([geo.latitude_place]'),num2str([geo.longitude_place]')]));
% geo = geo(m);
% clear b m n

% populate 'Souces' listbox
set(handles.Source_listbox,'String',{...
    'Paper use locations',...
    '',...
    '1 - Briquet Printed',...
    '2 - Likhachev',...
    '3 - Piccard Findbuch',...
    '4 - Piccard Online',...
    '5 - WILC',...
    '6 - WZMA',...
    '',...
    'Printing locations',...
    '',...
    '1 - ISTC',...
    '2 - GW',...
    '3 - WILC',...
    '',...
    'Repositories locations',...
    '',...
    '1 - Briquet',...
    '2 - GW',...
    '3 - Likhachev',...
    '4 - Piccard',...
    '5 - WILC',...
    '6 - WZMA',...
    '',...
    'Context locations',...
    '',...
    '1 - IBPH (Bibliography)',...
    '2 - Biraben (Plague)'});

% populate listboxes
set(handles.LocationsMenu_listbox,'Value',1,'String',{...
    'Locations',...
    'Regions level 3',...
    'Regions level 2',...
    'Regions level 1',...
    'Countries'});

index.countries = unique({geo.countries_names}');
set(handles.Countries_listbox,'String',[...
    '.. back';...
    ' ';...
    'Countries';...
    '';...
    index.countries]);

index.regions1 = unique({geo.regions1_names}');
% remove countries w/o higher nuts levels
if isempty( char(index.regions1(1)) )
    index.regions1 = index.regions1(2:length(index.regions1));
end
set(handles.Regions1_listbox,'String',[...
    '.. back';...
    ' ';...
    'Regions level 1';...
    '';...
    index.regions1]);

index.regions2 = unique({geo.regions2_names}');
% remove countries w/o higher nuts levels
if isempty( char(index.regions2(1)) )
    index.regions2 = index.regions2(2:length(index.regions2));
end
set(handles.Regions2_listbox,'String',[...
    '.. back';...
    ' ';...
    'Regions level 2';...
    '';...
    index.regions2]);

index.regions3 = unique({geo.regions3_names}');
% --- IF IDS INCORPORATED ---
% [index.regions3.name, m, n] = unique({geo.regions3_names}');
% t = unique({geo.regions3_id}');
% index.regions3.id = t(m);
% ---------------------------------------------------
% remove countries w/o higher nuts levels
if isempty( char(index.regions3(1)) )
    index.regions3 = index.regions3(2:length(index.regions3));
end
set(handles.Regions3_listbox,'String',[...
    '.. back';...
    ' ';...
    'Regions level 3';...
    '';...
    index.regions3]);

index.locations = unique({geo.locations_names}');
set(handles.Locations_listbox,'String',[...
    '.. back';...
    ' ';...
    'Locations';...
    '';...
    index.locations]);

% announce end of data loading
set(handles.Map_pushbutton,'Enable','on')
set(handles.LoadData_pushbutton,'String','Data loaded',...
    'ForegroundColor',TogOffCol,'BackgroundColor',TogOffBkCol);


% ----------------------------------
% MAP DATA - 1. DATASETS SELECTION
% ----------------------------------

% --- Executes on button press in Map_pushbutton.
function Map_pushbutton_Callback(hObject, eventdata, handles)
% hObject    handle to Map_pushbutton (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% wiz('Map_pushbutton_Callback',gcbo,[],guidata(gcbo))

% profile on -history
global geo index request requestOld header ...
    TogOnCol TogOnBkCol TogOffCol TogOffBkCol
header = 4;
if size(get(0,'Children'),1) >= 1 && ...
        get(handles.Superpose_radiobutton,'Value') == 1
    requestOld = request; % memorize last request
else
    requestOld = [];
end
set(handles.Map_pushbutton,'String','Mapping...',...
    'ForegroundColor',TogOnCol,'BackgroundColor',TogOnBkCol);
drawnow

% --- Select requested data ----------------------
% catch errors in selection
% ...

% unique({geo.source}')

k = [];
l1 = [];
% get selected items from listbox
selection = get(handles.Source_listbox,'Value');
l = size(selection,2);
if l > 0
    for m = 1:l
        switch selection(m)
            case 3
                k = [k; find(strcmp({geo.use_briquet}', 'x'))];
                l1 = [l1 'BP. '];
            case 4
                k = [k; find(strcmp({geo.use_likhachev}', 'x'))];
                l1 = [l1 'LKH. '];
            case 5
                k = [k; find(strcmp({geo.use_pf}', 'x'))];
                l1 = [l1 'PF. '];
            case 6
                k = [k; find(strcmp({geo.use_po}', 'x'))];
                l1 = [l1 'PO. '];
            case {7, 14}
                k = [k; find(strcmp({geo.use_wilc}', 'x'))];
                l1 = [l1 'WILC.'];
            case 8
                k = [k; find(strcmp({geo.use_wzma}', 'x'))];
                l1 = [l1 'WZMA. '];
            case 12
                k = [k; find(strcmp({geo.use_istc}', 'x'))];
                l1 = [l1 'ISTC. '];
            case 13
                k = [k; find(strcmp({geo.use_gw}', 'x'))];
                l1 = [l1 'GWP. '];
            case 18
                k = [k; find(strcmp({geo.repositories_briquet}', 'x'))];
                l1 = [l1 'BR. '];
            case 19
                k = [k; find(strcmp({geo.repositories_gw}', 'x'))];
                l1 = [l1 'GWR. '];
            case 20
                k = [k; find(strcmp({geo.repositories_likhachev}', 'x'))];
                l1 = [l1 'LR. '];
            case 21
                k = [k; find(strcmp({geo.repositories_piccard}', 'x'))];
                l1 = [l1 'PR. '];
            case 22
                k = [k; find(strcmp({geo.repositories_wilc}', 'x'))];
                l1 = [l1 'WILC. '];
            case 23
                k = [k; find(strcmp({geo.repositories_wzma}', 'x'))];
                l1 = [l1 'WZMA. '];
            case 27
                k = [k; find(strcmp({geo.context_ibph}', 'x'))];
                l1 = [l1 'IBPH. '];
            case 28
                k = [k; find(strcmp({geo.context_bir}', 'x'))];
                l1 = [l1 'BIR. '];
        end
    end
end
if isempty(l1)
    errormsg([...
        'Please make a selection '...
        'in the data source list.'])
    return
end
l1 = ['Source: ',l1];

if k > 0
    request = geo(unique(k)); % memorize request

    % get selected items from listbox
    nuts_level = get(handles.LocationsMenu_listbox,'Value');
    nuts_level = 5-nuts_level;
    if nuts_level == 4
        % --- Locations
        k = [];
        if strcmp(get(handles.LocationsMenu_listbox,'Visible'), 'on')
            k = 1:size(request,1); % all selected
            selection = get(handles.Locations_listbox,'Value');
            selection = selection(find(selection > header)) - header;
            selection_string = index.locations(selection);
        else
            selection = get(handles.Locations_listbox,'Value');
            selection = selection(find(selection > header)) - header;
            selection_string = index.locations(selection);
            l = size(selection,2);
            if l == 0
                errormsg('No location selected.')
                return
            elseif l == size(index.locations,1)
                k = 1:size(request,1); % all selected
            else
                for m = 1:l
                    k = [k; find(strcmp(...
                        {request.locations_names}', selection_string(m)))];
                end
            end
        end
    elseif nuts_level == 3
        % --- NUTS3
        k = [];
        selection = get(handles.Regions3_listbox,'Value');
        selection = selection(find(selection > header)) - header;
        selection_string = index.regions3(selection);
        % --- WHEN IDS INCORPORATED ---
        % selection_string = index.regions3.id(selection);
        l = size(selection,2);
        if l == 0
            errormsg('No location selected.')
            return
        elseif l == size(index.regions3,1)
            k = 1:size(request,1); % all selected
        else
            l = size(selection,2);
            for m = 1:l
                k = [k; find(strcmp(...
                    {request.regions3_names}', selection_string(m)))];
            end
        end
    elseif nuts_level == 2
        % --- NUTS2
        k = [];
        selection = get(handles.Regions2_listbox,'Value');
        selection = selection(find(selection > header)) - header;
        selection_string = index.regions2(selection);
        l = size(selection,2);
        if l == 0
            errormsg('No location selected.')
            return
        elseif l == size(index.regions2,1)
            k = 1:size(request,1); % all selected
        else
            l = size(selection,2);
            for m = 1:l
                k = [k; find(strcmp(...
                    {request.regions2_names}', selection_string(m)))];
            end
        end
    elseif nuts_level == 1
        % --- NUTS1
        k = [];
        selection = get(handles.Regions1_listbox,'Value');
        selection = selection(find(selection > header)) - header;
        selection_string = index.regions1(selection);
        l = size(selection,2);
        if l == 0
            errormsg('No location selected.')
            return
        elseif l == size(index.regions1,1)
            k = 1:size(request,1); % all selected
        else
            for m = 1:l
                k = [k; find(strcmp(...
                    {request.regions1_names}', selection_string(m)))];
            end
        end
    elseif nuts_level == 0
        % --- Countries
        k = [];
        selection = get(handles.Countries_listbox,'Value');
        selection = selection(find(selection > header)) - header;
        selection_string = index.countries(selection);
        l = size(selection,2);
        if l == 0
            errormsg('No location selected.')
            return
        elseif l == size(index.countries,1)
            k = 1:size(request,1); % all selected
        else
            for m = 1:l
                k = [k; find(strcmp(...
                    {request.countries_names}', selection_string(m)))];
            end
        end
    end

    if ~isempty(k)
        request = request(unique(k));
    else
        errormsg(['No data available for this selection.'])
        return
    end

    % draw map
    mapdata(request, nuts_level, selection_string, l1, handles)
    set(handles.Map_pushbutton,'String','Map',...
        'ForegroundColor',TogOffCol,'BackgroundColor',TogOffBkCol);
else
    set(handles.Map_pushbutton,'String','Map',...
        'ForegroundColor',TogOffCol,'BackgroundColor',TogOffBkCol);
    errormsg('No data available.')
    return
end
% profile viewer


% ----------------------------------
% MAP DATA - 2. MAPPING
% ----------------------------------
function mapdata(request, nuts_level, selection_string, l1, handles)

% --- Prepare map ----------------------
global geoshape lim target latestFig

% creates new window for drawing
if get(handles.NewWindow_radiobutton,'Value') == 1 || ...
    size(get(0,'Children'),1) == 1
    target = 1;

    hFig = get(0,'Children');
    screenpos = get(0,'ScreenSize');
    menupos = get(hFig(1),'Position');
    latestFig = figure('Name','Wiz',...
        'WindowButtonDownFcn','wiz(''popupinfo'')',...
        'MenuBar','none',...
        'ToolBar','figure',...
        'Color', [0.8, 0.75, 0.67],...
        'Position',...
        [menupos(1)+menupos(3)+50, menupos(2),...
        screenpos(3)-menupos(3)-150 menupos(4)-30]);
end
figure(latestFig)

% overwrites in the last window
if get(handles.Overwrite_radiobutton,'Value') == 1 || ...
        target == 1
    target = 2;

    cla % clear previous content

    % map extent & density
    if get(handles.Europe_radiobutton,'Value') == 1
        limlat = lim.lat.eu; limlon = lim.lon.eu;
        lat_ext = 'lat_eu'; lon_ext = 'lon_eu';
        grid_int = 5;
    else
        limlat = lim.lat.wd; limlon = lim.lon.wd;
        lat_ext = 'lat_wd'; lon_ext = 'lon_wd';
        grid_int = 10;
    end

    % projections: miller, eqdconicstd, behrmann
    axesm('MapProjection','miller',...
        'MapLatLimit', limlat, 'MapLonLimit', limlon,...
        'MLineLocation', grid_int, 'PLineLocation', grid_int,...
        'Frame', 'off', 'grid','off',...
        'FEdgeColor', 'none',...
        'FFaceColor', [0.8, 0.75, 0.67])
    set(gca,'Color',[0.8, 0.75, 0.67],...
        'XColor',[0.8, 0.75, 0.67], 'YColor',[0.8, 0.75, 0.67],...
        'Position',[0,0,1,1]);
    framem on
    
%     % debug (?): center map
%     k = get(gca,'xlim');
%     k = (abs(k(1))+k(2))/2;
%     set(gca,'Xlim', [-k,k] );
    
    % grid visibility
    if get(handles.Grid_checkbox,'Value') == 1
        gridm on, mlabel on, plabel on
    else
        gridm off, mlabel off, plabel off
    end

    % plot shapes
    % natural boundaries: continent shore, lakes, rivers
    % administrative boundaries: countries
    if get(handles.Lowres_radiobutton,'Value') == 1
        plotm(geoshape.shore_lo.(lat_ext), geoshape.shore_lo.(lon_ext), 'Color', [1 1 1])
        plotm(geoshape.border_lo.(lat_ext), geoshape.border_lo.(lon_ext), 'Color', [.63 .56 .45])
    else
        plotm(geoshape.border.(lat_ext), geoshape.border.(lon_ext), 'Color', [.63 .56 .45])
    end
    plotm(geoshape.river.(lat_ext), geoshape.river.(lon_ext),  'Color', [.875 .843 .792])
    plotm(geoshape.lake.(lat_ext), geoshape.lake.(lon_ext),  'Color', [.875 .843 .792])
    drawnow
end

% superposes datapoints on the last window
if get(handles.Superpose_radiobutton,'Value') == 1
    target = 3;
end

% --- Display data ----------------------
% aggregate data: remove duplicates
[b, m, n] = unique(cellstr(...
    [num2str([request.locations_latitude]'),...
    num2str([request.locations_longitude]')]));

latitude = [request(m).locations_latitude]';
longitude = [request(m).locations_longitude]';
set( gcf, 'Name', [ 'Wiz - ',num2str(size(m,1)),' locations selected' ] )
% xlabel([{[num2str(size(m,1)),' locations selected']};{l1}],...
%     'Color',[0 0 0])
clear b m n

% show nuts boundary
% (not for locations (handling not decided yet))
if get(handles.Regions_checkbox,'Value') == 1 && ...
    nuts_level ~= 4
    t = {'nuts0', 'nuts1', 'nuts2', 'nuts3'};
    t = char(t(nuts_level+1));
    for m = 1:size(selection_string,1)
        k = find(strcmp( geoshape.(t).name,...
            selection_string(m) ));
        p = geoshow(cell2mat(geoshape.(t).longitude(k)),...
            cell2mat(geoshape.(t).latitude(k)),...
            'DisplayType','polygon',...
            'FaceColor',[.73 .66 .55],...
            'EdgeColor',[.53 .46 .35]...
            );
        drawnow
    end
end

% figure out style of location-marker
mk = '.';
mkc = [1,0,0];
% superposes two datasets on same window
if get(handles.Superpose_radiobutton,'Value') == 1
    if isempty(findobj(latestFig,'Marker','.')) == 1 && ...
            isempty(findobj(latestFig,'Marker','o')) == 1
        mk = 'o';
        mkc = [0,0,1];
    elseif isempty(findobj(latestFig,'Marker','.')) == 0
        set(findobj(latestFig,'Marker','.'), 'Marker', 'o', 'Color', 'b')
        mk = '+';
        mkc = [1,0,0];
    elseif isempty(findobj(latestFig,'Marker','o')) == 0 && ...
            isempty(findobj(latestFig,'Marker','+')) == 1
        mk = '+';
        mkc = [1,0,0];
    elseif isempty(findobj(latestFig,'Marker','o')) == 0 && ...
            isempty(findobj(latestFig,'Marker','+')) == 0
        set(findobj(gcf,'Marker','+'), 'Marker', 'o', 'Color', 'b')
        mk = '+';
        mkc = [1,0,0];
    end
end

% plot location
l = plotm(latitude,longitude,mk','Color',mkc);

return
% --- Show statistics ----------------------
% point density
a = 50^2; % area of bins in km2
[lat2,lon2,num] = hista(latitude,longitude,a);
% q = scatterm(lat2,lon2,num,[0 0 1],'filled');

% i want a smoothed point density visualization
% i.e. like mountain isolines
% for both measurement of point densities
% and measurement of quantities per point per area
%
% do:
% - transform decimal coords to cccc real numbers
% - make a matrix of zeros min/max=lon/lat
% - insert ones in the indeces given by lon/lat
% - you got  a sparse matrix where you can measure 
%   the ponint density
% - move a circular window over the whole matrix
% - count the number of ones inside the window
% - add the result to a new matrix
% - you got a raster image with the point density
% - project the image on the map

% [x,y] = grn2eqa(lat2,lon2);
% [x,y] = grn2eqa(latitude,longitude);
% % [lat,lon] = eqa2grn(x, y);
% figure
% plot(x,y,'.r')
% axis square


% -------------------------------
% Interactive info
% -------------------------------
function popupinfo 
global request requestOld target
reqInfo = [request; requestOld];
latitude = [reqInfo.locations_latitude]';
longitude = [reqInfo.locations_longitude]';

hpt = []; hlab = []; htit = [];
while true
    [selected_lat,selected_lon,button] = inputm(1);
    if isempty(selected_lat)
        beep
        continue
    end
    if button == 3 % | isempty(selected_lat)
        delete(hpt, hlab, htit)  % delete last selection
        break % Finish selection process
    end
    delete(hpt, hlab, htit)  % delete last selection
    d = distance(latitude, longitude, selected_lat, selected_lon);
    k = find(d == min(d(:)));
    % handle duplicate coordinates
    k = k(1);
    
    % mark selected location
    if target == 3
        hpt = plotm(latitude(k),longitude(k),...
            'o','Color',[1 0 0],'MarkerSize',12);
    else
        hpt = plotm(latitude(k),longitude(k),...
            'o','Color',[0 0 1],'MarkerSize',12);
    end
    
%     % label next to the location
%     hlab = textm(latitude(k),longitude(k),...
%         [{[' ', reqInfo(k).locations_names,' (',...
%         num2str(latitude(k)),' lat ',...
%         num2str(longitude(k)),' lon',')']}],...

    % get variant names
    % search by name (variant: by coords)
    v = find(strcmp(...
        {reqInfo.locations_names}', reqInfo(k).locations_names));
    var = [];
    for m = 1:size(v,1)
        var = [ var,...
            {['{\fontsize{8}',reqInfo(v(m)).locations_names_source,'}']} ];
    end
    % catch no variants
    t = cell2mat(var(1));
    if size(var,1) == 1 && strcmp(t(14:end-1), reqInfo(k).locations_names)
        var = [];
    end
    
    % location legend
    leg = [ {['{\bf',reqInfo(k).locations_names,'}']},...
        {[num2str(latitude(k)),' lat, ',...
        num2str(longitude(k)),' lon']},...
        {reqInfo(k).countries_names} ];
    if isempty(reqInfo(k).regions1_names) == 0
        leg = [leg, {['  ',reqInfo(k).regions1_names]}];
    end
    if isempty(reqInfo(k).regions2_names) == 0
        leg = [leg, {['   ',reqInfo(k).regions2_names]}];
    end
    if isempty(reqInfo(k).regions3_names) == 0
        leg = [leg, {['    ',reqInfo(k).regions3_names]}];
    end
    if isempty(var) == 0
        leg = [leg, {' '}, var];
    end

    hlab = text(.1,.9,...
        leg,...
        'Units','normalized',...
        'FontSize',10,...
        'FontName','Arial Unicode MS',...
        'BackgroundColor',[0.8, 0.75, 0.67],...
        'LineWidth',1,...
        'Margin',5,...
        'horizontal','left','vertical','top'...
        );
    
%     ht = xlabel(reqInfo(k).name_place);
%     ht = title([{'Wiz'};...
%     htit = title([{[reqInfo(k).locations_names,' (',...
%         num2str(latitude(k)),' lat ',...
%         num2str(longitude(k)),' lon',')']}]);
    drawnow
end

% -------------------------------
% Call error
% -------------------------------
function errormsg(msg)
global TogOffCol TogOffBkCol
% catch error

set(findobj(gcf,'String','Mapping...'),...
    'String','Map',...
    'ForegroundColor',TogOffCol,'BackgroundColor',TogOffBkCol);
drawnow
hm = msgbox(msg,'Error');
set(hm,'Color',[0.8, 0.75, 0.67])
set(findobj(hm,'String','OK'),'Backgroundcolor',[0.8, 0.75, 0.67])


% ============================================================
% GUI components
% ============================================================

% --- Executes just before wiz is made visible.
function wiz_OpeningFcn(hObject, eventdata, handles, varargin)
% This function has no output args, see OutputFcn.
% hObject    handle to wiz_figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)
% varargin   command line arguments to wiz (see VARARGIN)

hw = msgbox('Reading map boundaries...','Loading','help');
delete(findobj(hw,'string','OK'))
drawnow

% Choose default command line output for wiz
handles.output = hObject;

% Update handles structure
guidata(hObject, handles);

% UIWAIT makes wiz wait for user response (see UIRESUME)
% uiwait(handles.wiz_figure);

% positions the menu window 50x 50y pixels from NW corner of the screen
screenpos = get(0,'ScreenSize');
menupos = get(gcf,'Position');
set(gcf,'Position',...
    [1, screenpos(4)-menupos(4)-50, menupos(3), menupos(4)])
findfigs

global geoshape lim
% save time by loading in advance boundary data

% % hi res political boundaries
% % --- Source code
% % put all coords in one vector
% B = load('worldhi');
% fn = fieldnames(B);
% geoshape.border.lat = [];
% geoshape.border.lon = [];
% % put all coords in one vector
% for k = 5:size(fn,1)
% geoshape.border.lat = [geoshape.border.lat; ...
%     NaN; B.(char(fn(k))).lat];
% geoshape.border.lon = [geoshape.border.lon; ...
%     NaN; B.(char(fn(k))).long];
% k
% end
% save worldhipol geoshape
% % --- 

% hi res political boundaries
load shapes/worldhipol;

% hi res nuts boundaries
try
    load shapes/nuts;
    geoshape.nuts0 = nuts.nuts0;
    geoshape.nuts1 = nuts.nuts1;
    geoshape.nuts2 = nuts.nuts2;
    geoshape.nuts3 = nuts.nuts3;
catch
    set(handles.Regions_checkbox,'Enable','off');
    set(handles.Regions_checkbox,'Value',0);
end
[num, txt, table] = xlsread('shapes/nuts.xls','NUTS0');
geoshape.nuts0.name = table(:,1);
[num, txt, table] = xlsread('shapes/nuts.xls','NUTS1');
geoshape.nuts1.name = table(:,1);
[num, txt, table] = xlsread('shapes/nuts.xls','NUTS2');
geoshape.nuts2.name = table(:,1);
[num, txt, table] = xlsread('shapes/nuts.xls','NUTS3');
geoshape.nuts3.name = table(:,1);
clear num txt table nuts

% low res political boundaries and shores
load shapes/worldlo POline;
geoshape.border_lo.lat = POline(1).lat;
geoshape.border_lo.lon = POline(1).long;
geoshape.shore_lo.lat = POline(2).lat;
geoshape.shore_lo.lon = POline(2).long;

R = shaperead('shapes/worldrivers');
L = shaperead('shapes/worldlakes');
geoshape.river.lat = [];
geoshape.river.lon = [];
for k = 1:size(R,1)
    geoshape.river.lat = [geoshape.river.lat, R(k).Y];
    geoshape.river.lon = [geoshape.river.lon, R(k).X];
end
geoshape.lake.lat = [];
geoshape.lake.lon = [];
for k = 1:size(L,1)
    geoshape.lake.lat = [geoshape.lake.lat, L(k).Y];
    geoshape.lake.lon = [geoshape.lake.lon, L(k).X];
end

% trim map
lim.lat.eu = [30 65]; lim.lon.eu = [-15 50];	% Europe
lim.lat.wd = [-90 90]; lim.lon.wd = [-180 180];	% World

limlat = lim.lat.eu; limlon = lim.lon.eu;
[geoshape.shore_lo.lat_eu, geoshape.shore_lo.lon_eu] = maptriml(...
    geoshape.shore_lo.lat, geoshape.shore_lo.lon,...
    limlat, limlon);
[geoshape.border_lo.lat_eu, geoshape.border_lo.lon_eu] = maptriml(...
    geoshape.border_lo.lat, geoshape.border_lo.lon,...
    limlat, limlon);
[geoshape.border.lat_eu, geoshape.border.lon_eu] = maptriml(...
    geoshape.border.lat, geoshape.border.lon,...
    limlat, limlon);
[geoshape.river.lat_eu, geoshape.river.lon_eu] = maptriml(...
    geoshape.river.lat, geoshape.river.lon,...
    limlat, limlon);
[geoshape.lake.lat_eu, geoshape.lake.lon_eu] = maptriml(...
    geoshape.lake.lat, geoshape.lake.lon,...
    limlat, limlon);

limlat = lim.lat.wd; limlon = lim.lon.wd;
[geoshape.shore_lo.lat_wd, geoshape.shore_lo.lon_wd] = maptriml(...
    geoshape.shore_lo.lat, geoshape.shore_lo.lon,...
    limlat, limlon);
[geoshape.border_lo.lat_wd, geoshape.border_lo.lon_wd] = maptriml(...
    geoshape.border_lo.lat, geoshape.border_lo.lon,...
    limlat, limlon);
[geoshape.border.lat_wd, geoshape.border.lon_wd] = maptriml(...
    geoshape.border.lat, geoshape.border.lon,...
    limlat, limlon);
[geoshape.river.lat_wd, geoshape.river.lon_wd] = maptriml(...
    geoshape.river.lat, geoshape.river.lon,...
    limlat, limlon);
[geoshape.lake.lat_wd, geoshape.lake.lon_wd] = maptriml(...
    geoshape.lake.lat, geoshape.lake.lon,...
    limlat, limlon);

delete(hw)

% --- Outputs from this function are returned to the command line.
function varargout = wiz_OutputFcn(hObject, eventdata, handles) 
% varargout  cell array for returning output args (see VARARGOUT);
% hObject    handle to wiz_figure
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% Get default command line output from handles structure
varargout{1} = handles.output;


% --- Executes on selection change in Source_listbox.
function Source_listbox_Callback(hObject, eventdata, handles)
% hObject    handle to Source_listbox (see GCBO)
% eventdata  reserved - to be defined in a future version of MATLAB
% handles    structure with handles and user data (see GUIDATA)

% --- Executes during object creation, after setting all properties.
function Source_listbox_CreateFcn(hObject, eventdata, handles)

% --- Executes on selection change in Locations_listbox.
function Locations_listbox_Callback(hObject, eventdata, handles)
if get(handles.Locations_listbox,'Value') == 1
    set(handles.Countries_listbox,'Visible','off')
    set(handles.Regions1_listbox,'Visible','off')
    set(handles.Regions2_listbox,'Visible','off')
    set(handles.Regions3_listbox,'Visible','off')
    set(handles.Locations_listbox,'Visible','off')
    set(handles.LocationsMenu_listbox,'Visible','on')
end

% --- Executes during object creation, after setting all properties.
function Locations_listbox_CreateFcn(hObject, eventdata, handles)

% --- Executes on selection change in Regions3_listbox.
function Regions3_listbox_Callback(hObject, eventdata, handles)
if get(handles.Regions3_listbox,'Value') == 1
    set(handles.Countries_listbox,'Visible','off')
    set(handles.Regions1_listbox,'Visible','off')
    set(handles.Regions2_listbox,'Visible','off')
    set(handles.Regions3_listbox,'Visible','off')
    set(handles.Locations_listbox,'Visible','off')
    set(handles.LocationsMenu_listbox,'Visible','on')
end

% --- Executes during object creation, after setting all properties.
function Regions3_listbox_CreateFcn(hObject, eventdata, handles)

% --- Executes on selection change in Regions2_listbox.
function Regions2_listbox_Callback(hObject, eventdata, handles)
if get(handles.Regions2_listbox,'Value') == 1
    set(handles.Countries_listbox,'Visible','off')
    set(handles.Regions1_listbox,'Visible','off')
    set(handles.Regions2_listbox,'Visible','off')
    set(handles.Regions3_listbox,'Visible','off')
    set(handles.Locations_listbox,'Visible','off')
    set(handles.LocationsMenu_listbox,'Visible','on')
end

% --- Executes during object creation, after setting all properties.
function Regions2_listbox_CreateFcn(hObject, eventdata, handles)

% --- Executes on selection change in Regions1_listbox.
function Regions1_listbox_Callback(hObject, eventdata, handles)
if get(handles.Regions1_listbox,'Value') == 1
    set(handles.Countries_listbox,'Visible','off')
    set(handles.Regions1_listbox,'Visible','off')
    set(handles.Regions2_listbox,'Visible','off')
    set(handles.Regions3_listbox,'Visible','off')
    set(handles.Locations_listbox,'Visible','off')
    set(handles.LocationsMenu_listbox,'Visible','on')
end

% --- Executes during object creation, after setting all properties.
function Regions1_listbox_CreateFcn(hObject, eventdata, handles)

% --- Executes on selection change in Countries_listbox.
function Countries_listbox_Callback(hObject, eventdata, handles)
if get(handles.Countries_listbox,'Value') == 1
    set(handles.Countries_listbox,'Visible','off')
    set(handles.Regions1_listbox,'Visible','off')
    set(handles.Regions2_listbox,'Visible','off')
    set(handles.Regions3_listbox,'Visible','off')
    set(handles.Locations_listbox,'Visible','off')
    set(handles.LocationsMenu_listbox,'Visible','on')
end

% --- Executes during object creation, after setting all properties.
function Countries_listbox_CreateFcn(hObject, eventdata, handles)

% --- Executes on selection change in LocationsMenu_listbox.
function LocationsMenu_listbox_Callback(hObject, eventdata, handles)
if get(handles.LocationsMenu_listbox,'Value') == 1
    set(handles.Countries_listbox,'Visible','off')
    set(handles.Regions1_listbox,'Visible','off')
    set(handles.Regions2_listbox,'Visible','off')
    set(handles.Regions3_listbox,'Visible','off')
    set(handles.Locations_listbox,'Visible','on')
    set(handles.LocationsMenu_listbox,'Visible','off')
elseif get(handles.LocationsMenu_listbox,'Value') == 2
    set(handles.Countries_listbox,'Visible','off')
    set(handles.Regions1_listbox,'Visible','off')
    set(handles.Regions2_listbox,'Visible','off')
    set(handles.Regions3_listbox,'Visible','on')
    set(handles.Locations_listbox,'Visible','off')
    set(handles.LocationsMenu_listbox,'Visible','off')
elseif get(handles.LocationsMenu_listbox,'Value') == 3
    set(handles.Countries_listbox,'Visible','off')
    set(handles.Regions1_listbox,'Visible','off')
    set(handles.Regions2_listbox,'Visible','on')
    set(handles.Regions3_listbox,'Visible','off')
    set(handles.Locations_listbox,'Visible','off')
    set(handles.LocationsMenu_listbox,'Visible','off')
elseif get(handles.LocationsMenu_listbox,'Value') == 4
    set(handles.Countries_listbox,'Visible','off')
    set(handles.Regions1_listbox,'Visible','on')
    set(handles.Regions2_listbox,'Visible','off')
    set(handles.Regions3_listbox,'Visible','off')
    set(handles.Locations_listbox,'Visible','off')
    set(handles.LocationsMenu_listbox,'Visible','off')
elseif get(handles.LocationsMenu_listbox,'Value') == 5
    set(handles.Countries_listbox,'Visible','on')
    set(handles.Regions1_listbox,'Visible','off')
    set(handles.Regions2_listbox,'Visible','off')
    set(handles.Regions3_listbox,'Visible','off')
    set(handles.Locations_listbox,'Visible','off')
    set(handles.LocationsMenu_listbox,'Visible','off')
end

% --- Executes during object creation, after setting all properties.
function LocationsMenu_listbox_CreateFcn(hObject, eventdata, handles)

% --- Executes on button press in NewWindow_radiobutton.
function NewWindow_radiobutton_Callback(hObject, eventdata, handles)
set(handles.NewWindow_radiobutton,'Value',1)
set(handles.Overwrite_radiobutton,'Value',0)
set(handles.Superpose_radiobutton,'Value',0)

% --- Executes on button press in Overwrite_radiobutton.
function Overwrite_radiobutton_Callback(hObject, eventdata, handles)
set(handles.NewWindow_radiobutton,'Value',0)
set(handles.Overwrite_radiobutton,'Value',1)
set(handles.Superpose_radiobutton,'Value',0)

% --- Executes on button press in Superpose_radiobutton.
function Superpose_radiobutton_Callback(hObject, eventdata, handles)
set(handles.NewWindow_radiobutton,'Value',0)
set(handles.Overwrite_radiobutton,'Value',0)
set(handles.Superpose_radiobutton,'Value',1)

% --- Executes on button press in Europe_radiobutton.
function Europe_radiobutton_Callback(hObject, eventdata, handles)
global lim
set(handles.Europe_radiobutton,'Value',1)
set(handles.World_radiobutton,'Value',0)
limlat = lim.lat.eu; limlon = lim.lon.eu;

figs = allchild(0);
figure(figs(end))
setm(gca,'MapLatLimit', limlat, 'MapLonLimit', limlon)

% --- Executes on button press in World_radiobutton.
function World_radiobutton_Callback(hObject, eventdata, handles)
set(handles.Europe_radiobutton,'Value',0)
set(handles.World_radiobutton,'Value',1)
% limlat = lim.lat.wd; limlon = lim.lon.wd;

% --- Executes on button press in Lowres_radiobutton.
function Lowres_radiobutton_Callback(hObject, eventdata, handles)
set(handles.Lowres_radiobutton,'Value',1)
set(handles.Highres_radiobutton,'Value',0)

% --- Executes on button press in Highres_radiobutton.
function Highres_radiobutton_Callback(hObject, eventdata, handles)
set(handles.Highres_radiobutton,'Value',1)
set(handles.Lowres_radiobutton,'Value',0)

% --- Executes on button press in Grid_checkbox.
function Grid_checkbox_Callback(hObject, eventdata, handles)
figs = allchild(0);
if get(handles.Europe_radiobutton,'Value') == 1
    grid_int = 5;
else
    grid_int = 10;
end

if get(handles.Grid_checkbox,'Value') == 1
    figure(figs(end))
    setm(gca,'MLineLocation', grid_int, 'PLineLocation', grid_int)
    gridm on, mlabel on, plabel on
else
    figure(figs(end))
    setm(gca,'MLineLocation', grid_int, 'PLineLocation', grid_int)
    gridm off, mlabel off, plabel off
end

% --- Executes on button press in Regions_checkbox.
function Regions_checkbox_Callback(hObject, eventdata, handles)

% --- Executes on button press in CloseWnds_pushbutton.
function CloseWnds_pushbutton_Callback(hObject, eventdata, handles)
latestFig = get(0,'Children');
if size(latestFig,1) > 1
    close(latestFig(2:end))
end

% --- Executes on button press in Help_pushbutton.
function Help_pushbutton_Callback(hObject, eventdata, handles)
web('help.txt','-browser');

% % --- Executes during object creation, after setting all properties.
% function Splash_axes_CreateFcn(hObject, eventdata, handles)
% imshow(imread('wiz-splash.png'));

% --- Executes on button press in Regions_pushbutton.
function Regions_pushbutton_Callback(hObject, eventdata, handles)
% --- BUTTON NOT VISIBLE ----
% boundary display is too slow to be of practical use
global geoshape

figs = allchild(0);
% catch no map yet generated
if size(figs,1) == 1
    return
end
% catch no nuts boundaries available
try
     geoshape.nuts0.longitude = geoshape.nuts0.longitude;
catch
    return
end
figure(figs(end))

title(' B U S Y  D R A W I N G ','FontWeight','bold',...
    'BackgroundColor',[1 0 0],'Color',[1 1 1]);
nn = {'nuts0', 'nuts1', 'nuts2', 'nuts3'};
% for n = size(nn,2):-1:1 % all nuts levels
for n = 2:2 % nuts1 only
    for k = 1:size(geoshape.(char(nn(n))).longitude)
        % display nuts boundaries
        geoshow(cell2mat(geoshape.(char(nn(n))).longitude(k)),...
            cell2mat(geoshape.(char(nn(n))).latitude(k)),...
            'DisplayType','line',...
            'Color',[.53+n/10 .46+n/10 .35+n/10]...
            );
        drawnow
    end
end
title('')

